#attach a pulser set to 4n

root = bespoke.get_root() + 48
scale = bespoke.get_scale()
chord = [scale[0] + root, scale[2] + root, scale[4] + root]

step = 0

def on_pulse():
   global step
   for p in chord:
      this.play_note(p, walk([60,30,10],step), 1.0/4)
   while True:
      newPitch = scale[random.randrange(len(scale))] + root + random.choice([-12,0,12])
      if not newPitch in chord:
         chord[random.randrange(len(chord))] = newPitch
         break
   step = step + 1
   
def walk(list, step):
   return list[step % len(list)]


#requires a drumsequencer named "drumsequencer"

import drumsequencer

seq = drumsequencer.get("drumsequencer")

def mutate():
   for step in range(16):
      for pitch in range(8):
         if random.random() < 0.1 and (pitch != 0 or step % 8 != 0):
            if random.random() < .15:
               seq.set(step, pitch, random.choice([80,127]))
            else:
               seq.set(step, pitch, 0)

def on_pulse():
   mutate()
   
on_pulse()


#attach a pulser set to 8n

lastPitch = -1

def on_pulse():
   global lastPitch
   root = bespoke.get_root() + 48
   scale = bespoke.get_scale()
   while (True):
      pitch = root + random.choice(scale) + random.choice([0,12])
      if pitch != lastPitch:
         break
   velocity = get_velocity()
   if abs(pitch - lastPitch) == 2:
      this.schedule_note(1.0/16, (pitch + lastPitch) / 2, int(velocity * .8), 1.0/17)
   this.schedule_note(1.0/8, pitch, velocity, 1.0/9)
   #if pitch % 12 == root % 12:
   #   this.schedule_note(1.0/8, pitch + 7, 70, 1.0/4)
   lastPitch = pitch
   
def get_velocity():
   velocities = [127,80,100]
   ret = velocities[(bespoke.get_step(8) + 1) % len(velocities)]
   this.output(str(ret))
   return ret


#requires a drumsynth named "drumsynth"

for i in range(8):
   if random.random() < .5:
      me.set("drumsynth~vol"+str(i), random.uniform(0,1))
      me.set("drumsynth~noise"+str(i), random.uniform(0,1))
      me.set("drumsynth~adsrtone"+str(i)+"A",random.uniform(1,10))
      me.set("drumsynth~adsrtone"+str(i)+"D",random.uniform(10,150))
      me.set("drumsynth~adsrnoise"+str(i)+"A",random.uniform(1,10))
      me.set("drumsynth~adsrnoise"+str(i)+"D",random.uniform(10,250))
      me.set("drumsynth~adsrfreq"+str(i)+"A",random.uniform(1,10))
      me.set("drumsynth~adsrfreq"+str(i)+"D",random.uniform(3,120))
      me.set("drumsynth~adsrfilter"+str(i)+"A",random.uniform(1,10))
      me.set("drumsynth~adsrfilter"+str(i)+"D",random.uniform(10,250))
      me.set("drumsynth~freqmax"+str(i),random.uniform(100,1000))
      me.set("drumsynth~freqmin"+str(i),random.uniform(10,100))
      me.set("drumsynth~cutoffmax"+str(i),random.uniform(100,10000))
      me.set("drumsynth~cutoffmin"+str(i),random.uniform(10,200))
      me.set("drumsynth~q"+str(i),random.uniform(.1,8))
      me.set("drumsynth~type"+str(i),random.choice([0,1,2,3]))


from __future__ import annotations

def get(path) -> oscoutput:
   pass

class oscoutput:
   def send_float(this, address, val):
      pass

   def send_int(this, address, val):
      pass

   def send_string(this, address, val):
      pass




#requires a drumsequencer named "drumsequencer"

import drumsequencer

d = drumsequencer.get("drumsequencer")

def randomVel():
   return random.choice([0,0,random.randint(1,127)])   

for row in range(8):
   for step in range(16):
      d.set(step, row, randomVel()) 
   for step in range(16):
      vel = d.get(step, row)
      if random.random() < .25:
         vel = randomVel()
      d.set(step+16, row, vel)

for step in range(32):
   if step % 4 == 0:
      d.set(step, random.choice([0,4]), 127)


#requires a sampleplayer named "sampleplayer"
#requires a drumplayer named "drumplayer"

import sampleplayer
import drumplayer

s = sampleplayer.get("sampleplayer")
d = drumplayer.get("drumplayer")

for i in range(8):
   s.set_cue_point(i, random.uniform(0,4),.1,1)
   d.import_sampleplayer_cue(s, i, i)


from __future__ import annotations

def get(path) -> notesequencer:
   pass

class notesequencer:
   def set_step(this, step, row, velocity = 127, length = 1.0):
      pass

   def set_pitch(this, step, pitch, velocity = 127, length = 1.0):
      pass




#a collection of tools for building named chords

progression = [('D2', 'M', 1, 1),
               ('F2', 'M', 1, 1),
               ('G2', 'min7', 0, 1),
               ('F#2', 'M7', 0, 1),]
               
qualities = {}
qualities[''] = [0]
qualities['M'] = [0, 4, 7]
qualities['m'] = [0, 3, 7]
qualities['dim'] = [0, 3, 6]
qualities['aug'] = [0, 4, 8]
qualities['M7'] = [0, 4, 7, 11]
qualities['min7'] = [0, 3, 7, 10]
qualities['D7'] = [0, 4, 7, 10]
qualities['mM7'] = [0, 3, 7, 11]
qualities['dim7'] = [0, 3, 6, 10]
 
def get_chord(root, quality, inversion):
   ret = []
   for i in range(len(qualities[quality])):
      pitch = bespoke.name_to_pitch(root) + qualities[quality][i]
      if i < inversion:
         pitch += 12
      ret.append(pitch)
   return ret

def add_chord(canvas, root, quality, inversion, position, length):
   for pitch in get_chord(root, quality, inversion):
      canvas.add_note(position, pitch, 127, length)
      
def play_chord(root, quality, inversion, length):
   for pitch in get_chord(root, quality, inversion):
      this.play_note(pitch, 127, length)




#attach a pulser set to 16n

def on_pulse():
   step = bespoke.get_step(16)
   if step % 16 in [0,4,8,11]:
      this.play_note(0,127,1.0/16)
   elif step % 16 in [4,11,14]:
      this.play_note(1,127,1.0/16)
   else:
      this.play_note(2,127,1.0/16)



def echo(pitch, velocity, repeats, length=1.0/16, loopLength=1, syncopateChance=0, pan=0, playChance=1, responseIndex=0, playFirstLoop=True, quantize=False):
   output = this.get_caller()
   for i in range(repeats):
      if i == 0 and playFirstLoop == False:
         continue
      if random.random() < playChance:
         quantizeShift = 0
         if quantize:
            quantizeSubdivision = 16.0
            quantizeShift = (round(bespoke.get_measure_time() * quantizeSubdivision) / quantizeSubdivision) - bespoke.get_measure_time()
         pos = i*loopLength + quantizeShift
         noteLength = length
         outputIndex = responseIndex
         if i == 0:
            outputIndex = 0
         canSyncopate = i > 0 and bespoke.get_step(32) % 4 == 0 or bespoke.get_step(32) % 4 == 3
         if canSyncopate and random.random() < syncopateChance:
            pos -= 1.0/16
            #if random.random() < .5:
            #   noteLength += 1.0/16
         output.schedule_note(pos, pitch, velocity, noteLength, pan=pan, output_index=outputIndex)



from __future__ import annotations

def get(path) -> linnstrument:
   pass

class linnstrument:
   def set_color(this, x, y, color):
      pass

   Off: ...
   Red: ...
   Yellow: ...
   Green: ...
   Cyan: ...
   Blue: ...
   Magenta: ...
   Black: ...
   White: ...
   Orange: ...
   Lime: ...
   Pink: ...



from __future__ import annotations

   def play_note(pitch, velocity, length=1.0/16.0, pan = 0, output_index = 0):
      pass

   def schedule_note(delay, pitch, velocity, length=1.0/16.0, pan = 0, output_index = 0):
      pass

   def schedule_note_msg(delay, pitch, velocity, pan = 0, output_index = 0):
      pass

   def schedule_call(delay, method):
      pass

   def note_msg(pitch, velocity, pan = 0, output_index = 0):
      pass

   def set(path, value):
      pass

   def schedule_set(delay, path, value):
      pass

   def get(path):
      pass

   def adjust(path, amount):
      pass

   def highlight_line(lineNum, scriptModuleIndex):
      pass

   def output(obj):
      pass

   def me():
      pass

   def stop():
      pass

   def get_caller():
      pass

   def set_num_note_outputs(num):
      pass

   def connect_osc_input(port):
      pass

   def send_cc(control, value, output_index = 0):
      pass




#requires a sampleplayer named "sampleplayer"

import sampleplayer

tempo = 99.5
secondsPerBeat = 60.0 / tempo
startOffsetSeconds = 3
startBeat = 0

sp = sampleplayer.get("sampleplayer")
for i in range(4):
   sp.set_cue_point(i, secondsPerBeat*(startBeat+i) + startOffsetSeconds, secondsPerBeat, 1)


from __future__ import annotations

def get(path) -> sampleplayer:
   pass

class sampleplayer:
   def set_cue_point(this, pitch, startSeconds, lengthSeconds, speed):
      pass

   def fill(this, data):
      pass

   def play_cue(this, cue, speedMult = 1, startOffsetSeconds = 0):
      pass

   def get_length_seconds(this, ):
      pass




#attach a pulser set to 1n
#requires the "schedule_sequence.py" script to be loaded

def on_pulse():
   pattern = [0,0,0,'',0,0,'',0,'',0,0,'']
   loop = (bespoke.get_measure()) % len(pattern)
   loop2 = (bespoke.get_measure() // 2) % len(pattern)
   loop3 = (bespoke.get_measure() // 3) % len(pattern)

   shifted = pattern[loop:] + pattern[:loop]
   shifted2 = pattern[loop2:] + pattern[:loop2]
   shifted3 = pattern[loop3:] + pattern[:loop3]
   
   schedule_seq(pattern, pan=-1, pitchOffset=4)
   schedule_seq(shifted, pan=1, pitchOffset=5)
   schedule_seq(shifted2, pan=-.5, pitchOffset=6)
   schedule_seq(shifted3, pan=.5, pitchOffset=7)
   schedule_seq([ 0, 0,'', 0])
   schedule_seq(['','', 1,''])
   
   #debug output
   this.output(loop)
   this.output(pattern)
   this.output(shifted)




from __future__ import annotations

def get(path) -> notecanvas:
   pass

class notecanvas:
   def add_note(this, measurePos, pitch, velocity, length):
      pass

   def clear(this, ):
      pass

   def fit(this, ):
      pass




#schedule a sequence with a tidal-like pattern notation

def schedule_seq(seq, pitchOffset=0, velocity=80, start=0, bars=1, sustain=1, pan=0, output=None, velocityRandomization=0, panRandomization=0):
   if output is None:
      output = this.get_caller()
   if not type(seq) is list:
      seq = [seq]
   if len(seq) == 0:
      return
   stepLength = float(bars) / len(seq)
   for i,entry in enumerate(seq):
      if entry != '':
         if type(entry) is tuple:
            for subentry in entry:
               schedule_seq(subentry, pitchOffset, velocity, start + stepLength * i, stepLength, sustain, pan, output)
         elif type(entry) is list:
            schedule_seq(entry, pitchOffset, velocity, start + stepLength * i, stepLength, sustain, pan, output)
         elif type(entry) is int or type(entry) is str:
            pitch = entry
            if type(entry) is str:
               pitch = bespoke.name_to_pitch(entry)
            output.schedule_note(start + stepLength * i, pitch + pitchOffset, int(velocity * random.uniform(1-velocityRandomization,1)), stepLength*sustain, pan=pan * random.uniform(1-panRandomization,1))


from __future__ import annotations

def get_measure_time():
   pass

def get_measure():
   pass

def reset_transport():
   pass

def get_step(subdivision):
   pass

def count_per_measure(subdivision):
   pass

def time_until_subdivision(subdivision):
   pass

def get_time_sig_ratio():
   pass

def get_root():
   pass

def get_scale():
   pass

def get_scale_range(octave, count):
   pass

def tone_to_pitch(index):
   pass

def name_to_pitch(noteName):
   pass

def pitch_to_name(pitch):
   pass

def pitch_to_freq(pitch):
   pass

def get_tempo():
   pass

def set_background_text(str, size=50, xPos = 150, yPos = 250, red = 1, green = 1, blue = 1):
   pass

def random(seed, index):
   pass

def get_modules():
   pass




#attach a pulser set to 16n, and some keyboard input

sequence = [60,60,60,60]
insertPos = 0

def on_pulse():
   this.play_note(sequence[bespoke.get_step(16)%len(sequence)],127,1.0/16)
   
def on_note(pitch, velocity):
   global sequence
   global insertPos
   if velocity > 0:
      sequence[insertPos % len(sequence)] = pitch
      insertPos += 1


#requires a midicontroller named "midicontroller"
#requires a m185sequencer named "m185sequencer"
#the controls are mapped to the default controls of the nanoKONTROL2 device

import midicontroller

m = midicontroller.get("midicontroller")

for i in range(8):
   m.set_connection(m.Control, i + 0, "m185sequencer~pitch"+str(i))
   m.set_connection(m.Control, i +16, "m185sequencer~pulses"+str(i))
   m.set_connection(m.Control, i +32, "m185sequencer~gate"+str(i), m.SetValue, 0)
   m.set_connection(m.Control, i +48, "m185sequencer~gate"+str(i), m.SetValue, 2)
   m.set_connection(m.Control, i +64, "m185sequencer~gate"+str(i), m.SetValue, 3)


#requires a notecanvas named "notecanvas"

import notecanvas

notes = [('C4', .125),
         ('A3', .125),
         ('G3', .125),
         ('A3', .125),
         ('F3', .125), 
         ('G3', .125),
         ('C4', .25)]

canvas = notecanvas.get("notecanvas")
canvas.clear()

pos = 0
for i in range(len(notes)):
   canvas.add_note(pos,bespoke.name_to_pitch(notes[i][0]),127,notes[i][1])
   pos = pos + notes[i][1]



#requires a notecanvas named "notecanvas"

import notecanvas

n = notecanvas.get("notecanvas")
scale = bespoke.get_scale_range(3,10)[::2]

n.clear()
t = 0.0
subdivision = 16
canvasLength = n.get("measures")

if canvasLength > 0:
   while t < canvasLength:
      length = 1/subdivision * random.randint(1,3)
      if t+length > canvasLength:
         length = canvasLength-t
      skip = False
      if length == 1/subdivision:
         skip = random.random() < .25
      if not skip:
         n.add_note(t, random.choice(scale), 127, length)
      t += length



#attach a pulser set to 16n

def on_pulse():
   step = bespoke.get_step(16)
   if (euclid(step,2,16)):
      me.play_note(0,127)
   if (euclid(step+4,3,16)):
      me.play_note(5,80)
   if (euclid(step,5,8)):
      me.play_note(2 if euclid(step,3,11) else 6,127)
      
def euclid(step, count, length):
   return math.floor(step*count/length) != math.floor((step-1)*count/length)


#adds digits in dozenal notation as variables
#this allows you to call me.play_note(d40, 127) to play the 0th note of the 4th octave

def dozenaldigit(decimal):
   if decimal == 11:
      return 'E'
   elif decimal == 10:
      return 'X'
   else:
      return str(decimal)

def dozenal(decimal):
   ret = dozenaldigit(decimal % 12)
   decimal //= 12
   while (decimal > 0):
      ret = dozenaldigit(decimal % 12) + ret
      decimal //= 12
   return ret

for i in range(128):
   globals()['d'+dozenal(i)] = i



from __future__ import annotations

def get(path) -> midicontroller:
   pass

class midicontroller:
   Note: ...
   Control: ...
   Program: ...
   PitchBend: ...
   Slider: ...
   SetValue: ...
   Toggle: ...
   Direct: ...
   SetValueOnRelease: ...
   Default: ...
   def set_connection(this, messageType, control, controlPath, controlType = Default, value = 0, channel = -1, page=0, midi_off=0, midi_on = 127, scale = false, blink = false, increment = 0, twoway = true, feedbackControl = -1, isPageless = false):
      pass

   def send_note(this, pitch, velocity, forceNoteOn = false, channel = -1, page = 0):
      pass

   def send_cc(this, ctl, value, channel = -1, page = 0):
      pass

   def send_program_change(this, program, channel = -1, page = 0):
      pass

   def send_pitchbend(this, bend, channel = -1, page = 0):
      pass

   def send_data(this, a, b, c, page = 0):
      pass

   def send_sysex(this, data, page = 0):
      pass

   def add_script_listener(this, script):
      pass

   def resync_controller_state(this, ):
      pass




from __future__ import annotations

def get(path) -> vstplugin:
   pass

class vstplugin:
   def send_cc(this, ctl, value, channel):
      pass

   def send_program_change(this, program, channel):
      pass

   def send_data(this, a, b, c):
      pass




from __future__ import annotations

def get(path) -> grid:
   pass

class grid:
   def set(this, col, row, value):
      pass

   def get(this, col, row):
      pass

   def set_grid(this, cols, rows):
      pass

   def set_label(this, row, label):
      pass

   def set_color(this, colorIndex, r, g, b):
      pass

   def highlight_cell(this, col, row, delay, duration, colorIndex=1):
      pass

   def set_division(this, division):
      pass

   def set_momentary(this, momentary):
      pass

   def set_cell_color(this, col, row, colorIndex):
      pass

   def get_cell_color(this, col, row):
      pass

   def add_listener(this, script):
      pass

   def clear(this, ):
      pass




import module

if 'osc' in globals():
   osc.delete()
osc = module.create("oscillator", 100, 600)
if 'pulser' in globals():
   pulser.delete()
pulser = module.create("pulser", 200, 300)
pulser.set("interval",5)

pulser.set_target(module.get("script"))
module.get("script").set_target(osc)
osc.set_target(module.get("gain"))  

def on_pulse():
   step = bespoke.get_step(8) % 8
   osc.set_position(150 * (step+1), 600)
   this.play_note(bespoke.get_root() + bespoke.get_scale()[step%7] + 36 + step / 7 * 12, 127, 1.0/8, pan=step / 4.0 - 1)
   


from __future__ import annotations

def get(path) -> module:
   pass

def create(moduleType, x, y):
   pass

class module:
   def set_position(this, x, y):
      pass

   def get_position_x(this, ):
      pass

   def get_position_y(this, ):
      pass

   def get_width(this, ):
      pass

   def get_height(this, ):
      pass

   def set_target(this, target):
      pass

   def set_target(this, targetPath):
      pass

   def get_target(this, ):
      pass

   def get_targets(this, ):
      pass

   def set_name(this, name):
      pass

   def delete(this, ):
      pass

   def set(this, path, value):
      pass

   def get(this, path):
      pass

   def adjust(this, path, amount):
      pass




#requires the "schedule_seq.py" script and the "chord_tools.py" script

Cm = tuple(get_chord('C3','m',0))
Fm = tuple(get_chord('F2','m',2))
Bb = tuple(get_chord('Bb2','M',0))
Gm = tuple(get_chord('G2','m',2))

sequence = [( Cm+('G#3','D3' ),  ['C2']*8  ),
            ( Fm+('',   'C#3'),  ['C#2']*8 ),
            ( Bb+('G#3','C3' ),  ['D2']*8  ),
            ( Gm+('G#3','Bb2'),  ['D2']*8  )]

schedule_seq(sequence, bars=8)
 


#attach a pulser set to 16n

scale = bespoke.get_scale_range(4,50)
chord = [0,2,4]
progression = [0,2,0,1]
steps = [0,4,8,12]
if not 'idx' in globals():
   idx = 0

def on_pulse():
   global idx
   if bespoke.get_step(16) % 16 in steps:
      if bespoke.get_step(16) % 16 == 0:
         idx = 0
      first = True
      for tone in chord:
         this.play_note(scale[tone+progression[idx]]+random.choice([0]),127,1.0/4)
      idx = (idx + 1) % len(progression)




#requires a midicontroller named "midicontroller"

import midicontroller

controller = midicontroller.get("midicontroller")
controller.add_script_listener(me.me())

def on_midi(messageType, control, value, channel):
   debug = str(messageType) + " " + str(control) + " " + str(value) + " " + str(channel)
   me.output(debug)


#attach a pulser set to 8n

scale = bespoke.get_scale_range(3,7)
random.shuffle(scale)

def on_pulse():
   step = bespoke.get_step(8)
   this.play_note(walk(scale,step), walk([110,30,60,80],step), 1.0/8)
   
def walk(list, step):
   return list[step % len(list)]